from .sillypass import (
    DIGITS,
    LETTERS,
    LOWERCASE,
    SYMBOLS,
    UPPERCASE,
    create_counts,
    create_password,
)


__all__ = [
    "create_counts",
    "create_password",
    "LETTERS",
    "UPPERCASE",
    "LOWERCASE",
    "DIGITS",
    "SYMBOLS",
]



import argparse
import secrets

from sillypass import SYMBOLS, create_counts, create_password


# main() is defined as a separate function so that it can be reached from the
# console script installed by the package.
def main() -> None:
    parser = create_parser()
    args = parser.parse_args()
    counts = create_counts(
        length=args.length,
        min_letters=args.min_letters,
        min_upper=args.min_upper,
        min_lower=args.min_lower,
        min_digits=args.min_digits,
        min_symbols=args.min_symbols,
        symbols=args.symbols,
    )
    print(create_password(counts, secrets.SystemRandom()))


def count(value: str) -> int:
    """
    Cast the value to a count (in other words, a nonnegative integer) and
    return the count.

    This function is intended to be used as an argparse type.
    """

    try:
        int_value = int(value)
    except ValueError:
        raise argparse.ArgumentTypeError(f"{value} is not an integer")

    if int_value < 0:
        raise argparse.ArgumentTypeError(f"{value} is negative")

    return int_value


def create_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        description="Generate random passwords meeting minimum character "
        "requirements.",
    )
    parser.add_argument(
        "--length", "-l",
        type=count,
        default=24,
        help="Number of characters to include in the password. "
        "Default: %(default)s",
    )
    parser.add_argument(
        "--min-letters",
        type=count,
        default=0,
        help="Minimum number of letters (upper or lowercase) to include in the "
        "password. Default: %(default)s",
    )
    parser.add_argument(
        "--min-upper",
        type=count,
        default=0,
        help="Minimum number of uppercase letters to include in the password. "
        "Default: %(default)s",
    )
    parser.add_argument(
        "--min-lower",
        type=count,
        default=0,
        help="Minimum number of lowercase letters to include in the password. "
        "Default: %(default)s",
    )
    parser.add_argument(
        "--min-digits",
        type=count,
        default=0,
        help="Minimum number of numeric digits to include in the password. "
        "Default: %(default)s",
    )
    parser.add_argument(
        "--min-symbols",
        type=count,
        default=0,
        help="Minimum number of symbols to include in the password. "
        "Default: %(default)s",
    )
    parser.add_argument(
        "--symbols",
        default=SYMBOLS,
        help="Symbols that are allowed to appear in the password. "
        "Default: %(default)s",
    )

    return parser


if __name__ == "__main__":
    main()



import random
import string
from typing import Dict, Mapping, Optional


LETTERS = string.ascii_letters
UPPERCASE = string.ascii_uppercase
LOWERCASE = string.ascii_lowercase
DIGITS = string.digits
SYMBOLS = string.punctuation


def create_password(counts: Mapping[str, int], rng: random.Random) -> str:
    """
    Randomly generate a string containing the given counts of characters.

    The `counts` parameter maps groups of characters to the exact number of
    times a character from that group should appear in the password. For
    example:

      count={"abc": 5, "de": 1}

    means the generated string should have 6 characters, 5 of which are chosen
    from "a", "b", or "c", and 1 of which is chosen from "d" or "e".
    """

    password = []

    for group in counts:
        count = counts[group]
        password += rng.choices(group, k=count)

    rng.shuffle(password)

    return ''.join(password)


def create_counts(
        length: int = 24,
        min_letters: int = 0,
        min_upper: int = 0,
        min_lower: int = 0,
        min_digits: int = 0,
        min_symbols: int = 0,
        symbols: str = SYMBOLS,
        remainder: Optional[str] = None,
) -> Dict[str, int]:
    """
    Given the specified minimum counts of each character group, return a data
    structure that can be passed to `create_password` as its `counts` argument.

    This is a convenience function that covers the requirements of the majority
    of services that place silly constraints on their passwords.

    `length` is the total length of the password. If the sum of the minimum
    counts is less than `length`, then the `remainder` character group pads the
    password to `length`. If `remainder` isn't provided, it defaults to the
    union of letters, digits, and symbols.
    """

    if length < 0:
        raise ValueError("`length` argument is negative")
    if min_letters < 0:
        raise ValueError("`min_letters` argument is negative")
    if min_upper < 0:
        raise ValueError("`min_upper` argument is negative")
    if min_lower < 0:
        raise ValueError("`min_lower` argument is negative")
    if min_digits < 0:
        raise ValueError("`min_digits` argument is negative")
    if min_symbols < 0:
        raise ValueError("`min_symbols` argument is negative")
    if min_symbols > 0 and symbols == "":
        raise ValueError("No characters provided in `symbols` argument.")

    min_length = (
        min_letters
        + min_upper
        + min_lower
        + min_digits
        + min_symbols
    )
    if min_length > length:
        raise ValueError(
            "Password length is not large enough to accommodate sum of minimum "
            f"counts: {length} < {min_length}."
        )

    counts = {}

    if min_letters > 0:
        counts[LETTERS] = min_letters
    if min_upper > 0:
        counts[UPPERCASE] = min_upper
    if min_lower > 0:
        counts[LOWERCASE] = min_lower
    if min_digits > 0:
        counts[DIGITS] = min_digits
    if min_symbols > 0:
        counts[symbols] = min_symbols

    if remainder is None:
        remainder = ''.join(set(LETTERS + DIGITS + symbols))

    min_remainder = length - min_length
    if min_remainder > 0:
        counts[remainder] = length - min_length

    return counts



from setuptools import find_packages, setup


setup(
    name="sillypass",
    version="1.0.0",
    description=(
        "Generate random passwords following silly rules like 1 letter,"
        " 1 number, 1 symbol"
    ),
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://gitlab.com/rjmorris/sillypass",
    author="Joey Morris",
    license="MIT",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Operating System :: OS Independent",
        "Intended Audience :: End Users/Desktop",
        "Topic :: Security",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    python_requires=">= 3.6",
    install_requires=[],
    entry_points={
        "console_scripts": [
            "sillypass=sillypass.__main__:main",
        ],
    },
)
